@@ -1,6 +1,7 @@
 import os
-import subprocess
+import time
 import paramiko
+import subprocess
 from log import log
 from datetime import datetime
 from func_timeout import func_set_timeout
@@ -34,10 +35,64 @@ class SSHClient(object):
         self.__username = username
         self.__password = password
         self.__connect_timeout = connect_timeout
-        self.__is_active = False
         self.__sftp = None
 
-    def __del__(self):
+    @property
+    def is_sshable(self):
+        ssh = paramiko.SSHClient()
+        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
+        try:
+            ssh.connect(
+                self.__ip,
+                port=self.__port,
+                username=self.__username,
+                password=self.__password,
+                look_for_keys=False,
+                allow_agent=False,
+                timeout=self.__connect_timeout
+            )
+            return True
+        except Exception as e:
+            log.warning(f"{self.__ip}:{self.__port} | server is not sshable.", exc_info=True)
+            return False
+        finally:
+            ssh.close()
+
+    def wait_for_sshable(self, timeout=60):
+        count=0
+        time_interval=5
+        while True:
+            count += 1
+            if self.is_sshable:
+                return True
+            if count > int(timeout/time_interval):
+                return False
+            time.sleep(time_interval)
+
+    def connect(self):
+        log.info(f" {self.__ip}:{self.__port} | begin to create ssh connect.")
+        try:
+            self.__ssh = paramiko.SSHClient()
+            self.__ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
+            self.__ssh.connect(
+                self.__ip,
+                port=self.__port,
+                username=self.__username,
+                password=self.__password,
+                look_for_keys=False,
+                allow_agent=False,
+                timeout=self.__connect_timeout
+            )
+            return True
+        except Exception as e:
+            log.error(f"{self.__ip}:{self.__port} | failed to ssh connect.", exc_info=True)
+            return False
+
+    def reconnect(self):
+        self.close()
+        return self.connect()
+
+    def close(self):
         try:
             self.__sftp.close()
         except:
@@ -60,7 +115,6 @@ class SSHClient(object):
                 while not channel.closed or channel.recv_ready() or channel.recv_stderr_ready():
                     end = datetime.now()
                     if (end-begin).total_seconds()>timeout:
-                        return_code=1
                         output=""
                         stderr=f"timeout to run cmd.{cmd}"
                     if channel.recv_ready():
@@ -88,23 +142,21 @@ class SSHClient(object):
                     f" {self.__ip}:{self.__port} | successful to run cmd {cmd}, output is:{std_output}.")
                 return ExecResult(exit_status, std_output, std_err)
         except Exception as e:
-            return ExecResult(255, "", str(e))
+            return ExecResult(1, "", str(e))
 
     def exec(self, cmd, promt_response=[], timeout=60):
         try:
-            if not self.__is_active:
+            if not self.__ssh:
                 if not self.reconnect():
                     raise RuntimeError("ssh transport is not active and failed to reconnect.")
             return self._exec(cmd,promt_response,timeout)
-        except FunctionTimedOut as e1:
-            log.error(f"when run cmd: {cmd}, meets exception.", exc_info=True)
-            return ExecResult(1, "", str(e1))
         except Exception as e:
             log.error(f"when run cmd: {cmd}, meets exception.", exc_info=True)
             return ExecResult(1, "", str(e))
 
     @func_set_timeout(int(os.environ.get("SSH_SCP_TO_REMOTE_TIMEOUT", 120)))
     def _scp_to_remote(self, local_path, remote_path):
+        self.__sftp = self.__ssh.open_sftp()
         log.info(
             f" {self.__ip}:{self.__port} | Begin to copy file from local {local_path} to remote host {remote_path}.")
         self.__sftp.put(local_path, remote_path)
@@ -123,6 +175,7 @@ class SSHClient(object):
             if not self.__is_active:
                 raise RuntimeError("ssh transport is not active.")
             os.environ["SSH_SCP_TO_REMOTE_TIMEOUT"] = str(timeout)
+            self.__sftp = self.__ssh.open_sftp()
             return self._scp_to_remote(local_path, remote_path)
         except RuntimeError:
             log.error(
@@ -144,6 +197,7 @@ class SSHClient(object):
             f" {self.__ip}:{self.__port} | Begin to copy file from remote {remote_path} to local host {local_path}.")
         if os.path.isfile(local_path):
             subprocess.run(['rm', '-rf', local_path], capture_output=True, text=True)
+        self.__sftp = self.__ssh.open_sftp()
         self.__sftp.get(remote_path, local_path)
         rs = subprocess.Popen(['ls', local_path], stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
         if rs.returncode == 0:
@@ -159,6 +213,7 @@ class SSHClient(object):
         try:
             if not self.__is_active:
                 raise RuntimeError("ssh transport is not active.")
+            self.__sftp = self.__ssh.open_sftp()
             os.environ["SSH_SCP_FILE_TO_LOCAL_TIMEOUT"] = str(timeout)
             return self._scp_file_to_local(remote_path, local_path)
         except RuntimeError:
@@ -175,47 +230,6 @@ class SSHClient(object):
             log.error(f"when scp from remote {remote_path} to local {local_path}, meets exception.", exc_info=True)
             return False
 
-    def reconnect(self):
-        try:
-            self.__sftp.close()
-        except:
-            pass
-        try:
-            self.__ssh.close()
-        except:
-            pass
-        return self.connect()
+    def __del__(self):
+        self.close()
 
-
-    def connect(self):
-        log.info(f" {self.__ip}:{self.__port} | begin to create ssh connect.")
-        try:
-            self.__ssh = paramiko.SSHClient()
-            self.__ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-            self.__ssh.connect(
-                self.__ip,
-                port=self.__port,
-                username=self.__username,
-                password=self.__password,
-                look_for_keys=False,
-                allow_agent=False,
-                timeout=self.__connect_timeout
-            )
-            stdin, stdout, stderr = self.__ssh.exec_command("ls /|grep root")
-            std_output = stdout.read().decode()
-            if "root" in std_output:
-                log.info(f"{self.__ip}:{self.__port} | successful to ssh:OK.")
-                self.__sftp = self.__ssh.open_sftp()
-                self.__is_active = True
-                return True
-            else:
-                log.warning(f"{self.__ip}:{self.__port} | failed to ssh. stderr is {stderr.read().decode()}.")
-                self.__is_active = False
-        except Exception as e:
-            log.error(f"{self.__ip}:{self.__port} | failed to ssh connect.", exc_info=True)
-            self.__is_active = False
-            return False
-
-    @property
-    def is_active(self):
-        return self.__is_active
